
outdir='HOME1/rna-seq-pH-1/DE1/deseq2-smic/fgsea1'
design= 'HOME1/rna-seq-pH-1/DE1/deseq2-smic/dsSmic2_quantile0.25-15-reads.design.txt'
fpms=   'HOME1/rna-seq-pH-1/DE1/deseq2-smic/dsSmic2_quantile0.25-15-reads.merged-DE-FPM.txt'
pair=list()  # condition , contorl
pair[[1]]=c('pH8.2','pH7.6')
pair[[2]]=c('pH7.8','pH7.6')
# when fpms_filter == TRUE , pair_pval should be prepared
pair_pval=c("pH_pH7.8_vs_pH7.6.padj" ,"pH_pH7.8_vs_pH7.6.padj")
pair_logfc=c("pH_pH7.8_vs_pH7.6.log2FoldChange" ,"pH_pH7.8_vs_pH7.6.log2FoldChange")

gos='HOME1/WD1/data/dbs/Symbiodinium-microadriaticum-NCBI/uniprot/uniprot-symbiodinium-microadriaticum.tab.GOs.molten_gene-to-GOterms.txt'
gos_types=c("GO_BP", "GO_MF", "GO_CC") # "KEGG"
fpms_filter=FALSE

calculate_logFC_fromFpm=FALSE

#hclust_method="ward.D"

######################################################
# load GO and expression data:

library(data.table)

setwd(outdir)

fpms0=read.csv(fpms,sep="\t",stringsAsFactors=F)
fpms1=fpms0[,grepl('^FPM\\.',names(fpms0),perl=T)]

fpms1_logfc = as.data.frame(fpms0[,pair_logfc])
names(fpms1_logfc)=pair_logfc
rownames(fpms1_logfc) = fpms0$ID
if(fpms_filter){ fpms1_padj = as.data.frame(fpms0[,pair_pval]); names(fpms1_padj)=pair_pval }
rownames(fpms1) = fpms0$ID
design1=read.csv(design,sep="\t",stringsAsFactors=F)
names1=sapply(names(fpms1),function(x) gsub('FPM\\.X?','',x,perl=T))
sum(names1==rownames(design1))
sum(names1!=rownames(design1))
#if(sum(names1 != rownames(design1)) !=0 ){ cat("unequal names of samples"); }
groups1 = design1[,2]

gos1=read.csv(gos,sep="\t",stringsAsFactors=F)

#genesSets0 = aggregate(gos1$ID,by=list(gos1$term_id),function(x){paste(unique(x),collapse=';')})
#names(genesSets0) = c('term_id', 'genes')
gos1.dt=as.data.table(gos1)
genesSets0 = as.data.frame(gos1.dt[,.(genes = paste(unique(ID),collapse=';')),by = term_id])
genesSets0 = genesSets0[order(genesSets0$term_id),]
write.table(genesSets0, file = paste0(outdir,'/geneSets.txt'),row.names=F,quote=F,sep = "\t")

genesSets1 = do.call(list,apply(genesSets0,1,function(x) strsplit(x[2],';')[[1]]))
names(genesSets1) = genesSets0$term_id

id_to_name = gos1[!duplicated(gos1$term_id),c('term_id','term_name')]

save.image(paste0(outdir,'/fgsea-data.RData'))

##################################
#  fgsea:

library(fgsea)
library(ggplot2)

gseaRes2_=list()
for(i in 1:length(pair)){
    condition = pair[[i]][1]
    control = pair[[i]][2]
    pair1 = paste0(condition,' - ',control)
    cat("compare ",pair1,"\n")
    cont = names(fpms1)[groups1 == control]
    cond = names(fpms1)[groups1 == condition]
    if((length(cond)<1) | (length(cont)<1)){cat("error: empty group found\n"); break}
    log2FC = fpms1_logfc[,pair_logfc[i]]
    names(log2FC)=rownames(fpms1_logfc)
    if(fpms_filter){
        pval   = fpms1_padj[,pair_pval[i]]
        fpms2 =  fpms1[!is.na(pval),c(cont,cond)]
        log2FC=log2FC[!is.na(pval)]
    }else{
        fpms2=fpms1[,c(cont,cond)]
    }
    if(calculate_logFC_fromFpm==T){
        log2FC=apply(fpms2,1,function(x) log2(mean(x[cond]+1)/mean(x[cont]+1)))
    }
    score1=log2FC
    gseaRes = fgsea(pathways = genesSets1, stats = score1,minSize=5,maxSize=500,nperm=10000)
    gseaRes1 = gseaRes[gseaRes$padj<=0.15,]
    gseaRes1 = merge(id_to_name,gseaRes1,by.x='term_id',by.y='pathway',all.x=F,all.y=T)
    sets_with_enriched_genes = genesSets1[names(genesSets1) %in% gseaRes1[,'term_id']]
    dbSize = data.frame(term_id=names(sets_with_enriched_genes),db_size=sapply(sets_with_enriched_genes,length))
    gseaRes1 = merge(gseaRes1,dbSize,by='term_id',all.x=T,all.y=F)
    gseaRes1 = gseaRes1[order(gseaRes1$padj),]
    gseaRes2 = gseaRes1
    gseaRes2$leadingEdge_size = sapply(gseaRes2$leadingEdge,function(x) length(x))
    gseaRes2$leadingEdge = sapply(gseaRes2$leadingEdge,function(x) paste(x,collapse=';'))
    gseaRes2$direction = sapply(gseaRes2$ES,function(x) ifelse(x>0,'up',ifelse(x<0,'down','no')))
    
    write.table(gseaRes2, file = paste0(outdir,'/',pair1,'_enrichment_fgsea.txt'),row.names=F,quote=F,sep = "\t")
    gseaRes2_[[i]]=gseaRes2
    
    if(nrow(gseaRes2)>0){
        pdf(paste0(outdir,'/',pair1,'_fgsea.pdf'))
        for(j in 1:min(nrow(gseaRes2),1000)){
            z=as.character(gseaRes2[j,'term_id'])
            zz=as.character(gseaRes2[j,'term_name'])
            print(plotEnrichment(genesSets1[[z]],score1) + labs(title=paste(z,zz)))
        }
        dev.off()
    }
    
    sink(file=paste0(outdir,'/',pair1,'_enrichment_fgsea.log.txt'),append=ifelse(i==1,FALSE,TRUE))
    cat("pair = ",pair1,"\n")
    if(fpms_filter){cat("pvalue = ",pair_pval[i],"\n")}
    cat("terms = ",nrow(gseaRes2))
    sink()
}

save.image(paste0(outdir,'/fgsea-data.2.RData'))

######################################
#  functional groups clustering:

library(ggplot2)
library(dendextend)
library(plyr)
library(circlize)
library(gridExtra)
library(data.table)
library(stringdist)

dist_funct=function(v,func){
    n=length(v)
    m=matrix(data=NA,nrow=n,ncol=n)
    for(j in 1:n){
        if(j<n){
            for(k in (j+1):n){
                dist = func(v[j],v[k])
                m[j,k]= dist
                m[k,j]= dist
            }
        }
    }
    return(m)
}

dist_overlap=function(x,y){
    x=strsplit(x,';')[[1]]
    y=strsplit(y,';')[[1]]
    z=length(intersect(x,y))
    return(1-(z/min(length(x),length(y))))
}

dist_jaccard=function(x,y){
    x=strsplit(x,';')[[1]]
    y=strsplit(y,';')[[1]]
    return(1-length(intersect(x,y))/length(union(x,y)))
}

dist_Levenshtein=function(x,y){
    return(adist(c(x),c(y))[1:1]/max(nchar(x),nchar(y)))
}

pairwise_dists=function(ids,dist_mat_){
    dist_mat=as.matrix(dist_mat_)
    ids_hash=list()
    rownames0=rownames(dist_mat_)
    for(j in 1:length(rownames0)){ids_hash[[rownames0[j]]]=j}
    dists=c()
    n=length(ids)
    for(j in 1:n){
        if(j<n){
            for(k in (j+1):n){
                dists=c(dists,dist_mat[ids_hash[[ids[j]]],ids_hash[[ids[k]]]])
            }
        }
    }
    return(dists)
}

elbow=function(x,y,k,start){
    z=rep(0,times=(start-1))
    for(i in (start):length(y)){z = c(z,(y[i] / y[i-k])/(x[i] - x[i-k]))}
    return(z)
}

colors1=c('black','blue','brown','burlywood4','cadetblue','chartreuse4','chocolate3','chocolate4','coral3',
          'blue4','blueviolet','brown1','darkgoldenrod4','deepskyblue4','firebrick4','darkslategrey','purple','red',
          'green','gray',rainbow(n=100))

gseaRes3=list()
#jacc_ = list()
leve_ = list()
d_ = list()
for(i in 1:length(pair)){
    condition = pair[[i]][1]
    control = pair[[i]][2]
    direction=c('up','down')
    for(direction1 in direction){
        gseaRes2__0 = gseaRes2_[[i]][gseaRes2_[[i]]$direction==direction1,]
        pair1 = paste0(condition,'-',control,'_',direction1)
        cat("compare ",pair1,"\n")
        if(nrow(gseaRes2__0)>=10){
            gseaRes2__0$term_name = apply(gseaRes2__0[,c('term_id','term_name')],1,function(x) ifelse(x[2]=='' | is.na(x[2]),x[1],x[2]))
            leve = dist_funct(gseaRes2__0$term_name,dist_Levenshtein)
            #jacc = dist_funct(gseaRes2__0$leadingEdge,dist_jaccard)
            overlaps = dist_funct(gseaRes2__0$leadingEdge,dist_overlap)
            gseaRes2__=gseaRes2__0
            
            if(nrow(gseaRes2__)>=10){
                
                d = dist_funct(gseaRes2__$leadingEdge,dist_overlap)
                d_[[i]]=d
                #jacc_[[i]]=jacc
                leve_[[i]]=leve
                colnames(d)=gseaRes2__$term_name
                rownames(d)=gseaRes2__$term_name
                hc = hclust(as.dist(d),method="ward.D2") #hclust_method)
                hcd = as.dendrogram(hc)
                
                scores=c()
                max1=min(20,max(as.integer(nrow(gseaRes2__)/10),4))
                cuts0=1:max1
                cutr1=list()
                cutr_clusters1=list()
                for(u in 1:length(cuts0)){
                    cat(u,"\n")
                    cutr1[[u]] = cutree(tree=hc,k=cuts0[u])
                    cutr_freqs = as.data.frame(table(cutr1[[u]]))
                    cutr_clusters1[[u]]=data.frame(term_name=names(cutr1[[u]]),cluster=cutr1[[u]])
                    cutr_clusters1[[u]]=cutr_clusters1[[u]][order(cutr_clusters1[[u]]$cluster),]
                    cutr_clusters1_ = as.data.table(cutr_clusters1[[u]])
                    fun1=function(x,d){w=pairwise_dists(x,d); return(sum(w<0.5))}
                    ag1=cutr_clusters1_[,.(x=fun1(term_name,d)),by=cluster]
                    scores=c(scores,sum(ag1$x))
                }
                
                idx0=as.integer(max1/2)
                cutr=cutr1[[idx0]]
                cuts=cuts0[idx0]
                cutr_clusters=cutr_clusters1[[idx0]]
                cutr_tightClusters=cutr_clusters1[[length(cutr_clusters1)]]
                names(cutr_tightClusters) = c('term_name','tightCluster')
                cutr_tight = cutr1[[length(cuts0)]]
                cutrColors = mapvalues(cutr,from=unique(cutr),to=colors1[1:length(unique(cutr))])
                fcColors =   mapvalues(gseaRes2__$direction,from=c('up','down','no'),
                                       to=c('green','red','gray'),warn_missing=F)
                cutr_tight_colors = mapvalues(cutr_tight,from=unique(cutr_tight),to=colors1[1:length(unique(cutr_tight))])
                
                cutr_levels = c(as.integer(idx0/2),idx0,max1)
                for(c in 1:length(cutr_levels)){
                    w = cutr_clusters1[[cutr_levels[c]]]
                    names(w)=c('term_name',paste0('tree_depth.',cutr_levels[c]))
                    if(c==1){ cutr_clusters_ = w
                    }else{ cutr_clusters_ = merge(cutr_clusters_,w,by='term_name',all.x=T,all.y=T) }
                }
                gseaRes3[[i]] = merge(cutr_clusters_,gseaRes2__,by='term_name',all.x=F,all.y=T)
                mtch=match(gseaRes2__$term_id , gseaRes3[[i]]$term_id)
                gseaRes3[[i]] = gseaRes3[[i]][mtch,]
                gseaRes3[[i]] = gseaRes3[[i]][order.dendrogram(hcd),]
                write.table(gseaRes3[[i]], file = paste0(outdir,'/',pair1,'_enrichment_fgsea.clusters.txt'),row.names=F,quote=F,sep = "\t")
                
                pdf(paste0(outdir,'/',pair1,'_fgsea-tree.pdf'))
                par(bg='white')
                par(mar=c(1,1,1,1))
                #par(mar=c(1,1,1,25)) # for rectangular tree
                g=hcd %>% dendextend::set("leaves_pch", 19) %>%  dendextend::set("leaves_cex", 0.3) %>%
                    dendextend::set("labels_cex", 0.25) %>%
                    dendextend::set( "leaves_col", cutr_tight_colors[order.dendrogram(hcd)]) %>%
                    dendextend::set( "labels_colors", cutrColors[order.dendrogram(hcd)]) %>%
                    circlize_dendrogram(labels_track_height=0.3,facing='outside')
                    #plot(horiz=T) # rectangular tree
                title(main="complete tree", outer=F,cex.main = 0.5)
                print(g)
                dev.off()
            }
        }
    }
}

save.image(paste0(outdir,'/fgsea-data.3.RData'))