script_params='design/design1.STAR.sh'
genome1='$HOME1/$WD1/data/stylophora-genome/GCF_002571385.1_Stylophora_pistillata_v1_genomic.fna'
star_genome_dir='$HOME1/$WD1/data/stylophora-genome/STAR'
gtf='$HOME1/$WD1/data/stylophora-genome/GCF_002571385.1_Stylophora_pistillata_v1_genomic.gff.gtf'
outdir='$HOME1/rna-seq-pH-1/mapping'
gtf_idattr='gene_id'
pairing="PE"
# min(14, log2(GenomeLength)/2 - 1) # 400megabase genome size gives 13, default  14
genomeSAindexNbases=13

###########################################

function assert_ { rc=$?; if [[ $rc != 0 ]]; then echo 'exit !!!!'; exit $rc; fi }

. $script_params
assert_

if [ ! -z ${CONDA+x} ]; then
	if [ $1 -eq 1 ]; then
		if [ ! -d "$star_genome_dir" ]; then mkdir "$star_genome_dir"; assert_; fi

		sbatch --mem=128000 -N1 -n20 --ntasks-per-node=20 \
		    -o $genome1.star.out -e $genome1.star.err \
		    -p hive1d,hive7d,hiveunlim,queen,preempt1d,preempt7d,preempt31d,ckpthp1d,ckpthp7d,ckpthp31d,ckptdell1d,ckptdell7d,ckptdell31d \
		    --wrap "
					if [ ! -z ${CONDA+x} ]; then
						function assert_ { rc=$?; if [[ $rc != 0 ]]; then echo 'exit !!!!'; exit $rc; fi }
						. \"$CONDA\"; assert_
						conda activate star2.5; assert_
						STAR --runMode genomeGenerate --runThreadN 20 --genomeFastaFiles \"$genome1\" --genomeDir \"$star_genome_dir\" \
							--sjdbGTFfile \"$gtf\" --outFileNamePrefix \"$star_genome_dir/star\" --genomeSAindexNbases $genomeSAindexNbases
						assert_
						conda deactivate; assert_
					else
						echo \"CONDA global variable doesn't exist\"; assert_
					fi
					"
	elif [ $1 -ge 2 ] && [ $1 -le 3 ] && [ -d "$outdir" ] && [ -f "$gtf" ]; then
		for i in ${!title[@]}; do
            t="$outdir/${title[i]}"
		    echo "title = ""${title[i]}"
		    echo "for = ""${for1[i]}"
            if [ ! -z "${rev1[i]}" ]; then echo "rev = ${rev1[i]}"; fi
            if [ $1 -eq 3 ]; then
                if [ $pairing == "SE" ]; then
                    if [ ! -z "${for1[i]}" ] && [ ! -z "${title[i]}" ]; then
                        if [ -f "${for1[i]}" ]; then
                            sbatch --mem=128000 -N1 -n20 --ntasks-per-node=20 \
                                -o "$t".star.out -e "$t".star.err \
                                -p hive1d,hive7d,hiveunlim,queen,preempt1d,preempt7d,preempt31d,ckpthp1d,ckpthp7d,ckpthp31d,ckptdell1d,ckptdell7d,ckptdell31d \
                                --wrap "echo SE
                                        function assert_ { rc=$?; if [[ $rc != 0 ]]; then echo 'exit !!!!'; exit $rc; fi }
                                        if [ ! -z ${CONDA+x} ]; then
                                            . \"$CONDA\"; assert_
                                            conda activate star2.5; assert_
                                            STAR --readFilesIn \"${for1[i]}\" --outFileNamePrefix \"$t.star\" --runThreadN 20 --genomeDir \"$star_genome_dir\" \
                                                 --outSAMtype BAM SortedByCoordinate --quantMode GeneCounts
                                            assert_
                                            conda deactivate; assert_
                                        else
                                            echo \"CONDA global variable doesn't exist\"; assert_
                                        fi
                                        "
                        fi
                    fi
                elif [ $pairing == "PE" ]; then
                    if [ ! -z "${for1[i]}" ] && [ ! -z "${rev1[i]}" ] && [ ! -z "${title[i]}" ]; then
                        if [ -f "${for1[i]}" ] && [ -f "${rev1[i]}" ]; then
                            sbatch --mem=128000 -N1 -n20 --ntasks-per-node=20 \
                                -o "$t".star.out -e "$t".star.err \
                                -p hive1d,hive7d,hiveunlim,queen,preempt1d,preempt7d,preempt31d,ckpthp1d,ckpthp7d,ckpthp31d,ckptdell1d,ckptdell7d,ckptdell31d \
                                --wrap "echo PE
                                        function assert_ { rc=$?; if [[ $rc != 0 ]]; then echo 'exit !!!!'; exit $rc; fi }
                                        if [ ! -z ${CONDA+x} ]; then
                                            . \"$CONDA\"; assert_
                                            conda activate star2.5; assert_
                                            STAR --readFilesIn \"${for1[i]}\" \"${rev1[i]}\" --outFileNamePrefix \"$t.star\" --runThreadN 20 --genomeDir \"$star_genome_dir\" \
                                                 --outSAMtype BAM SortedByCoordinate --quantMode GeneCounts
                                            assert_
                                            conda deactivate; assert_
                                        else
                                            echo \"CONDA global variable doesn't exist\"; assert_
                                        fi
                                        "
                        fi
                    fi
                fi
		    fi
		    echo "----------------"
		    #break
		done
	elif [ $1 -eq 4 ]; then
		python gene-info-from-gtf_1.py --gtf "$gtf" --gene_regexpr "gene\d+" --output_file "$gtf.gene-length.txt"
		#python gene-info-from-gtf_1.py --gtf "temp/test_gene-info-from-gtf_1.gtf" --gene_regexpr "ENSRNOG\d+" --output_file "temp/test_gene-info-from-gtf_1.gtf.length.txt"
	elif [ $1 -eq 5 ]; then
		mkdir "$outdir/multiqc"
		assert_
		sbatch --mem=128000 -N1 -n20 --ntasks-per-node=20 \
			-o "$outdir/multiqc/multiqc".out -e "$outdir/multiqc/multiqc".err \
			-p hive1d,hive7d,hiveunlim,queen,preempt1d,preempt7d,preempt31d,ckpthp1d,ckpthp7d,ckpthp31d,ckptdell1d,ckptdell7d,ckptdell31d \
				--wrap "
						function assert_ { rc=$?; if [[ $rc != 0 ]]; then echo 'exit !!!!'; exit $rc; fi }
						if [ ! -z ${CONDA+x} ]; then
							. \"$CONDA\"; assert_
							conda activate rnaseq_denovo_2; assert_
							cd $outdir/multiqc
							multiqc $outdir
							assert_
							conda deactivate; assert_
						else
							echo \"CONDA global variable doesn't exist\"; assert_
						fi
						"
	fi
else
	echo "CONDA global variable doesn't exist"; assert_
fi

